{% load silica %}
{% load silica_scripts %}
{% silica_scripts %}
<link rel="icon" type="image/png" href="data:image/png;base64,iVBORw0KGgo=">

{% silica 'silica.tests.test_cache_doesnt_persist.CachePersistTestComponent' %}
