class SilicaCacheError(Exception):
    pass


class SetPropertyException(Exception):
    pass


class ComponentNotValid(Exception):
    pass


class SilicaSetPropertyException(Exception):
    pass


class SingleRootNodeError(Exception):
    pass
